/*
 * ShareTunes main JavaScript
 *
 * copyright (c) nonn et twk
 */

var CONTENTS_FOLDER = system.widgetDataFolder + '/Contents';

// include_once which manages include
if (!this.include_once) include_once = function(path){
	include_once._included || (include_once._included = {});
	if (include_once._included[path]) return;
	include(path);
	include_once._included[path] = true;
}; // include_once

include_once(CONTENTS_FOLDER + '/log4js.js');
include_once(CONTENTS_FOLDER + '/log4js_ywe.js');
include_once(CONTENTS_FOLDER + '/compat.js');
include_once(CONTENTS_FOLDER + '/prototype.js');
include_once(CONTENTS_FOLDER + '/base64.js');
include_once(CONTENTS_FOLDER + '/util.js');
include_once(CONTENTS_FOLDER + '/Twitter.js');

var logger = new Log4js.Logger('ShareTunes');
logger.addAppender(new YahooWidgetPrintAppender(logger));
logger.setLevel(Log4js.Level.INFO);


// @see iTunes COM SDK
var ITSourceKind = {RadioTuner:6, SharedLibrary:7};
var ITTrackKind = {File:1};

// Yahoo! Widget Engine 
// system.platform
var SYSTEM_PLATFORM = {MACINTOSH:'macintosh', WINDOWS:'windows'};
var ITUNES_TRACK_TYPE = {SHARED_LIBRARY:'shared library'};


// class as Yahoo Widget iTunes and iTunes COM object wrapper
function ShareTunes(){
	// @see mouseEventTimer.interval (sec)
	this.popupMouseDownMsec = 50; // const

	this.yahoo = null;
	this.scpt = null; // Apple Script
	this.com = null;
	this.disabledCom = null; // when deffered
	this.xml = null;
	this.amazon = new Amazon('nonnettwkdnsa-22', '1SRXPR21BWJPR1CM1JR2');

	this.lastTrack = null;

	// circular array
	this.playHistory = [];
	this.playHistoryPointer = 0;

	this.amazonInfo = '';
	this.wikipediaInfo = '';

	// preference settings
	this.twitter = null;
	this.twitterAutoUpdate = false;
	this.webBrowser = null;

	// fixed path for "who listens to my tracks?"
	this.handle = 'c:/programs/handle/handle.exe';
	this.nkf = 'c:/programs/nkf/nkf.exe';

	this.hideItunesOnGainFocus = true;
	this.updateViewsFrequenceSec = null;
	this.maxTracksInfoCount = 15;
} // ShareTunes
ShareTunes.prototype = {
	runItunes : function(){
		if (GetItunes().getCom())
			return true;

		if (GetItunes().getAppleScript())
			return GetItunes().getAppleScript().runItunes();
		
		logger.error('runItunes failed');
		return false;
	}
	,getYahoo : function getYahoo(){
		if (!this.yahoo) this.yahoo = iTunes
		return this.yahoo;
	}
	,getAppleScript : function(){
		if (this.scpt === null)
		{
			var scpt = false;
			if (system.platform.toLowerCase() == SYSTEM_PLATFORM.MACINTOSH)
			{
				scpt = {
					runItunes : function() {
						return appleScript('tell application "iTunes" to run', 30);
					}
					// @return boolean
					,isItunesVisible : function() {
						return 'true' == appleScript('tell application "iTunes" to get visible of item 1 of browser windows');
					}
					,setItunesVisibility : function(value) {
						return appleScript('tell application "iTunes" to set visible of item 1 of browser windows to ' 
							+ (value ? 'true' : 'false'));
					}
				}; // scpt
			}
			this.scpt = scpt;
		}
		return this.scpt;
	} // getAppleScript
	// @return comObject or false
	,getCom : function getCom(){
		if (this.com === null)
		{
			var com = false;
			if (system.platform.toLowerCase() == SYSTEM_PLATFORM.WINDOWS)
			{
				com = COM.createObject("iTunes.Application");
				if (com)
				{
					try
					{
						com.BrowserWindow.Minimized = true;
						COM.connectObject( com, "iTunes_" );// to enable getting some callbacks
						logger.info('COM initialized (But no way to know if connectObject is succeeded).');
					}
					catch (e)
					{
						com = false;
						logger.info('COM initialized but an exception is thrown.  Try again later.');
					}
				}
				else
				{
					com = null; // to try again later
					logger.info('COM initialization failed.');
				}
			}
			this.com = com;
		}
		return this.com;
	}
	,releaseCom : function releaseCom(){
		if (this.disabledCom)
		{
			COM.disconnectObject(this.disabledCom);
			this.disabledCom = null;
		}
		else if (this.com)
		{
			COM.disconnectObject(this.com);
			this.com = null;
		}
	}
	,getXml : function(){
		if (!this.xml)
		{
			this.xml = new function(){
				this.doc = null;
				this.fileName = system.widgetDataFolder + '/ShareTunes Music Libray.xml';

				// initial file
				if (filesystem.isDirectory(this.fileName))
					logger.error('unexpected error');
				if (!filesystem.itemExists(this.fileName))
					filesystem.copy(CONTENTS_FOLDER + '/ShareTunes Music Libray.xml', this.fileName);

				// load
				this.doc = XMLDOM.parse(filesystem.readFile(this.fileName));
				logger.info("Loaded file '" + this.fileName + "'");
				// logger.debug(this.doc.toXML());

				this.save = function(){
					// logger.debug(this.doc.toXML());
					filesystem.writeFile(this.fileName, this.doc.toXML());
					logger.info("Saved file '" + this.fileName + "'");
				};
				this.rate = function(track, rating){
					try
					{
						var xpath = '//Tracks/Track['
								+ 'Name="' + track.Name.replace('"', '\\"', 'g') + '"'
								+ ' and Artist="' + track.Artist.replace('"', '\\"', 'g') + '"'
								+ ']';
						// logger.debug(xpath);
						var trackNode = this.doc.evaluate(xpath).item(0);
						if (!trackNode)
						{
							trackNode = this.createTrackNode(track);

							var xmlRate = this.doc.createElement('Rating');
							xmlRate.appendChild(this.doc.createTextNode(rating));
							trackNode.appendChild(xmlRate);

							var tracks = this.doc.evaluate('//Tracks').item(0);
							tracks.appendChild(trackNode);
						}
						else
						{
							var xmlRate = trackNode.evaluate('Rating').item(0);
							xmlRate.replaceChild(this.doc.createTextNode(new String(rating)), xmlRate.firstChild);
						}
						this.save();
					}catch(e){
						if (e.code)
							logger.error("DOMException caught.  " + e + ", code=" + e.code);
						else
							throw e; // should not happen
					}
				};
				this.getRating = function(track){
					var xpath = 'string(//Tracks/Track['
							+ 'Name="' + track.Name.replace('"', '\\"', 'g') + '"'
							+ ' and Artist="' + track.Artist.replace('"', '\\"', 'g') + '"'
							+ ']/Rating)';
					var rating = parseInt(this.doc.evaluate(xpath));
					if (isNaN(rating)) rating = 0;
					return rating;
				};
				this.createTrackNode = function(track){
					var newId = 1 + parseInt(this.doc.evaluate('string(//Tracks/Track[last()]/@key)'));
					if (isNaN(newId)) newId = 1;

					var newTrack = this.doc.createElement('Track');
					newTrack.setAttribute('key', newId);

					var name = this.doc.createElement('Name');
					name.appendChild(this.doc.createTextNode(track.Name));
					newTrack.appendChild(name);

					var artist = this.doc.createElement('Artist');
					artist.appendChild(this.doc.createTextNode(track.Artist));
					newTrack.appendChild(artist);

					var trackId = this.doc.createElement('TrackID');
					trackId.appendChild(this.doc.createTextNode(newId));
					newTrack.appendChild(trackId);

					return newTrack;
				};
			}(); // this.xml
		}
		return this.xml;
	}
	// open url with the preferred or system browser
	,openUrl : function openUrl(url){
		if (this.webBrowser)
		{
			var commandLine
				= system.platform.toLowerCase() == SYSTEM_PLATFORM.MACINTOSH
				? 'open ' + this.webBrowser + ' "' + url + '"'
				: '"' + this.webBrowser + '" "' + url + '"';
			runCommandInBg(commandLine, commandLine); // tag is same as commandLine
		}
		else
			openURL(url);
	}

	,playPause : function(){
		this.getYahoo().playPause();
	}
	,backTrack : function(){
		this.getYahoo().backTrack();
	}
	,nextTrack : function(){
		this.getYahoo().nextTrack();
	}
	// iTunes does not handle rate for remote tracks
	// so we try to do it with an original setting file.
	,rate : function(rating){
		// no track is selected
		if (this.getYahoo().trackTitle == '')
			return;

		// AppleScript Execution Error may be outputed to the console (but thrown nothing)
		this.getYahoo().trackRating = rating;
		if (this.getYahoo().trackRating != rating 
			|| (rating == 0 && this.getYahoo().trackType == ITUNES_TRACK_TYPE.SHARED_LIBRARY))
		{
			if (this.getCom())
				this.getXml().rate(this.getCom().CurrentTrack, rating);
			else
				logger.info('remote track rating not updated.');
		}
		this.updateRatingView();
	}
	// @return 0 to 100.  0 if no currentTrack
	,getCurrentRating : function(){
		var rating = this.getYahoo().trackRating;
		if (rating == 0 && this.getCom() && this.getYahoo().trackType == ITUNES_TRACK_TYPE.SHARED_LIBRARY)
			rating = this.getXml().getRating(this.getCom().CurrentTrack);
		return rating;
	}

	,isMute: function(){ return this.getCom() ? this.getCom().Mute : system.mute; }
	// @param value boolean
	,setMute: function(value){
		if (this.isMute() == value)
			return;

		if (this.getCom())
			this.getCom().Mute = value;
		else
			system.mute = value;
	}
	,toggleMute: function(){
		return this.setMute(!this.isMute());
	}

	,getVolume: function(){ return this.getYahoo().volume; }
	,setVolume: function(value){ 
		if (this.getYahoo().volume != value) this.getYahoo().volume = value; }

	,isShuffle : function(){ return this.getYahoo().shuffle; }
	// @param value boolean
	,setShuffle : function(value){ 
		if (this.isShuffle() != value) this.getYahoo().shuffle = value; }
	,toggleShuffle: function(elementName){
		return this.setShuffle(!this.isShuffle());
	}

	// @return 'off', 'one' or 'all'
	,getRepeatMode : function(){ return this.getYahoo().repeatMode; }
	,setRepeatMode : function(value){ return this.getYahoo().repeatMode = value; }
	,toggleRepeatMode : function(){
		this.setRepeatMode({off:'all', all:'one', one:'off'}[this.getRepeatMode()] || 'all');
	}

	// @return empty if not com
	,getSources: function(){
		return this.getCom() ? this.getCom().Sources : {Count:0};
	}

	,toggleItunes : function(){
		logger.debug('toggleItunes');

		if (this.isItunesVisible())
			this.hideItunes();
		else
			this.showItunes();
	}
	// @return false if windows and cannot get COM at that moment
	,isItunesVisible : function(){
		return (this.getCom() && !this.getCom().BrowserWindow.Minimized)
			|| (this.getAppleScript() && this.getAppleScript().isItunesVisible());
	}
	,showItunes : function(){
		if (this.getAppleScript())
		{
			this.getAppleScript().setItunesVisibility(true);
			return;
		}

		if (!this.getCom())
			return;

		this.getCom().BrowserWindow.Visible = true;
		// minimize and then un-minimize lets the window foreground
		this.getCom().BrowserWindow.Minimized = true;
		this.getCom().BrowserWindow.MiniPlayer = false;
		this.getCom().BrowserWindow.Minimized = false;
	}
	,hideItunes : function(){
		if (this.getAppleScript())
		{
			this.getAppleScript().setItunesVisibility(false);
			return;
		}

		if (!this.getCom())
			return;

		this.getCom().BrowserWindow.Minimized = true;
	}

	// @return string or empty string if iTunes seems busy
	,getCurrentTrackDetailInfo : function GetCurrentTrackDetailInfo()
	{
		var s = [];

		// iTunes seems busy
		if (!this.getYahoo().trackTitle)
			return '';

		addFormatProperty(s, 'Title', this.getYahoo().trackTitle);
		var trackLengthSec = this.getYahoo().trackLength;
		if (trackLengthSec > 0)
		{
			var trackLengthMin = Math.floor(trackLengthSec/60);
			trackLengthSec = trackLengthSec - trackLengthMin * 60;
			if (trackLengthSec < 10) trackLengthSec = '0' + trackLengthSec;
			var trackLengthString = trackLengthMin + ':' + trackLengthSec;
			addFormatProperty(s, 'Length', trackLengthString);
		}

		addFormatProperty(s, 'Artist', this.getYahoo().trackArtist);
		addFormatProperty(s, 'Album', this.getYahoo().trackAlbum);

		if (this.getCom())
		{
			var track  = this.getCom().CurrentTrack;
			addFormatProperty(s, 'Genre', track.Genre);
			addFormatProperty(s, 'Year', track.Year);
			addFormatProperty(s, 'BPM', track.BPM);
			// addFormatProperty(s, 'SampleRate', (track.SampleRate/1000) + 'KHz');
			if (this.getYahoo().trackType != ITUNES_TRACK_TYPE.SHARED_LIBRARY)
			{
				var playedDate = '';
				if (track.PlayedDate)
				{
					playedDate = new Date(track.PlayedDate)
					playedDate = 'M/d h:mm'
						.replace('M', playedDate.getMonth() + 1)
						.replace('d', playedDate.getDate())
						.replace('h', playedDate.getHours())
						.replace('mm', (playedDate.getMinutes() < 9 ? '0' : '') + playedDate.getMinutes());
				}
				
				addFormatProperty(s, 'Played', 
					track.PlayedCount + (playedDate ? ' (' + playedDate + ')' : '') );
			}
		}

		addFormatProperty(s, 'Type', 
			this.getCom() ? track.KindAsString : widget.getLocalizedString(this.getYahoo().trackType));

		if (this.getYahoo().trackType != ITUNES_TRACK_TYPE.SHARED_LIBRARY)
			addFormatProperty(s, 'Rating', this.getYahoo().trackRating);

		return s.join("\n");

        // name is used via getLocalizedString
		function addFormatProperty(arry, name, value, pred){
			if (!pred) pred = function(value){return value != '';}
			if (!pred(value)) return;
			arry.push(widget.getLocalizedString(name) + ': ' + value);
		}
	} // getCurrentTrackDetailInfo

	// Get text for "Now Playing"
	,getCurrentTrackInfo : function GetCurrentTrackInfo()
	{
	    return this.getTrackInfo();
	}
	// @param track COM ITrack if not specified, current track is used
	,getTrackInfo : function (track)
	{
		if (!track) track = {Name:this.getYahoo().trackTitle, Artist:this.getYahoo().trackArtist};
		
		var a = [];

		var trackTitle = track.Name;
		trackTitle != '' && a.push(trackTitle);

		var trackArtist = track.Artist;
		trackArtist != '' && a.push(trackArtist);

		return a.join(' / ');
	}
	// @param relativeIndex -1 for backTrack, +1 for nextTrack etc..
	,getRelativeTrackInfo : function(relativeIndex)
	{
		var s = '';
		do {
			// we need com
			if (!this.getCom()) break;
			if (this.getRepeatMode() == 'one') break;

			var currentTrack = this.getCom().CurrentTrack;
			if (!currentTrack) break;

			var playlist = this.getCom().CurrentPlaylist;
			if (!playlist) break;

			var tracks = playlist.Tracks;
			if (!tracks) break;

			var playOrderIndex = currentTrack.PlayOrderIndex + relativeIndex;
			if (playOrderIndex < 1 || playOrderIndex > tracks.Count)
				break;

			var track = tracks.ItemByPlayOrder(playOrderIndex);
			if (!track)
				break;

			s += this.getTrackInfo(track);
		} while (false);
		return s;
	}
	,getBackTrackInfo : function()
	{
		return this.getRelativeTrackInfo(-1);
	}
	,getNextTrackInfo : function()
	{
		return this.getRelativeTrackInfo(+1);
	}
	,getTwitterUpdateMessage : function(track)
	{
		var trackTitle = track ? track.Name : this.getYahoo().trackTitle;
		var trackArtist = track ? track.Artist : this.getYahoo().trackArtist;

		var s = "";
		s += trackTitle;
		if (trackTitle != '' && trackArtist != '')  s += ' by ';
		s += trackArtist;

		return s != '' ? 'Listening to ' + s : '';
	}

	,selectPlayListFromSource: function(source)
	{
		if (!this.getCom()) return;

		var items = [];

		var playLists = source.Playlists;
		for (var iPlayList = 1; iPlayList <= playLists.Count; ++iPlayList)
		{
			var playList = playLists.Item(iPlayList);
			// logger.debug(playList.Name + ':' + playList.Size);
			if (!playList.Size) continue;

			var item = new MenuItem();
			item.title = source.Name + ' > ' + playList.Name;
			// logger.debug(item.title);

			with({playList:playList})
			{
				item.onSelect = function(){
					if (!playList)
					{
						logger.warn('The selected playList has gone');
						return;
					}
					// play a random track from the list
//					try {
//						playList.Tracks.Item(random(1, playList.Tracks.Count)).Play();
//					} catch(e) {
//						logger.debug(e);
						playList.PlayFirstTrack();
//					}
					GetItunes().updatePlayerStatus();
					GetItunes().updateTrackInfoView();
					// repeat and shuffle status are changed according to a playlist
					GetItunes().updateRepeatView();
					GetItunes().updateShuffleView();
				};
			}

			items.push(item);
		}
		// logger.debug(items.length);
		if (items.length > 1)
			popupMenu(items, system.event.hOffset, system.event.vOffset);
		else
			items[0].onSelect();
	}

	,selectPlayList: function()
	{
		if (!this.getCom()) return;

		var items = [];

		var sources = GetItunes().getSources();
		for (var iSource = 1; iSource <= sources.Count; ++iSource)
		{
			var source = sources.Item(iSource);
			// logger.debug(source.Name + ':' +source.Playlists.Count+'*'+source.Capacity);

			// cannot start to play radio
			if (source.Kind == ITSourceKind.RadioTuner)
				continue;

			var item = new MenuItem();
			item.title = source.Name;

			with({source:source})
			{
				item.onSelect = function(){
					if (!source)
					{
						logger.warn('The selected source has gone');
						return;
					}
					GetItunes().selectPlayListFromSource(source);
				};
			}

			items.push(item);
		}
		logger.debug('popup items length=' + items.length);

		if (items.length > 1)
			popupMenu(items, system.event.hOffset, system.event.vOffset);
		else
			items[0].onSelect();
	}

	// popup the real play history
	,popupPlayHistoryInfo : function()
	{
		// recent goes first
		var tracks = this.playHistory.slice(this.playHistoryPointer)
			.concat(this.playHistory.slice(0, this.playHistoryPointer))
			.reverse();

		if (!tracks || tracks.length < 2) return false;

		// @see popupTracksInfo

		var iBegin = 0, iEnd = tracks.length - 1;

		logger.debug('popupPlayHistoryInfo ' + tracks.length + ',' + iBegin + ',' + iEnd);

		var items = [];

		// loop [iBegin..iEnd]
		for (var iTrack = iBegin, step = (iBegin < iEnd ? 1 : -1), l = iEnd + step;
			iTrack != l;
			iTrack += step)
		{
			var track = tracks[iTrack];

			var item = this.createMenuItem(track);
			items.push(item);
		}
		logger.debug('popup items length=' + items.length);

		popupMenu(items, system.event.hOffset, system.event.vOffset);
		return true;
	}

	// @return true if shows popup
	,popupNextTracksInfo : function()
	{
		if (!this.getCom()) return false;

		var currentPlayOrderIndex = null;
		var currentTrack = this.getCom().CurrentTrack;
		if (currentTrack)
			currentPlayOrderIndex = currentTrack.PlayOrderIndex;

		var playlist = this.getCom().CurrentPlaylist;
		if (!playlist) return false;
		var tracks = playlist.Tracks;
		if (!tracks || tracks.length < 2) return false;

		var iBegin = currentPlayOrderIndex;
		var iEnd = Math.min(tracks.Count, iBegin + this.maxTracksInfoCount - 1);

		this.popupTracksInfo(tracks, iBegin, iEnd, currentPlayOrderIndex);
		return true;
	}

	// iBegin > iEnd is supported.  in such case, iBegin goes upper and iEnd goes lower
	// @param iBegin
	// @param iEnd
	,popupTracksInfo : function(tracks, iBegin, iEnd, currentPlayOrderIndex)
	{
		logger.debug('popupTracksInfo ' + tracks.Count + ',' + iBegin + ',' + iEnd + ',' + currentPlayOrderIndex);
		var items = [];

		// loop [iBegin..iEnd]
		for (var iTrack = iBegin, step = (iBegin < iEnd ? 1 : -1), l = iEnd + step;
			iTrack != l;
			iTrack += step)
		{
			var track = tracks.ItemByPlayOrder(iTrack);

			var item = this.createMenuItem(track);
			if (iTrack == currentPlayOrderIndex)
				item.enabled = false;

			items.push(item);
		}
		logger.debug('popup items length=' + items.length);

		if (items.length > 1)
			popupMenu(items, system.event.hOffset, system.event.vOffset);
		else
			items[0].onSelect();
	}

	,createMenuItem : function(track)
	{
		var item = new MenuItem();
		item.title = this.getTrackInfo(track);

		with({track:track})
		{
			item.onSelect = function(){
				if (!track)
				{
					logger.warn('The selected track has gone');
					return;
				}
				logger.debug(track.Name);
				track.Play();
				// Need GetItunes() call in the event handler
				GetItunes().updatePlayerStatus();
				GetItunes().updateTrackInfoView();
			};
		}
		return item;
	}

	,selectTrack: function()
	{
		if (!this.getCom()) return;

		var items = [];

		var playlist = this.getCom().CurrentPlaylist;
		if (!playlist) return;

		var tracks = playlist.Tracks;
		if (!tracks) return;

		var currentPlayOrderIndex = null;
		var currentTrack = this.getCom().CurrentTrack;
		if (currentTrack)
			currentPlayOrderIndex = currentTrack.PlayOrderIndex;

		var maxMenuItemCount = 20; // for performance
		var iBegin = 1;
		var iEnd = tracks.Count;
		if (iEnd - iBegin > maxMenuItemCount)
		{
			if (!currentPlayOrderIndex)
				iEnd = iBegin + maxMenuItemCount - 1;
			else
			{
				iBegin = currentPlayOrderIndex - Math.floor(maxMenuItemCount / 2);
				if (iBegin < 1)
				{
					iBegin = 1;
					iEnd = iBegin + maxMenuItemCount - 1;
				}
				else
				{
					iEnd = iBegin + maxMenuItemCount - 1;
					if (iEnd > tracks.Count)
					{
						iEnd = tracks.Count;
						iBegin = iEnd - maxMenuItemCount + 1;
					}
				}
			}
		}
		logger.debug("Showing " + iBegin + '-' + iEnd + " from " + tracks.Count);

		for (var iTrack = iBegin, l = iEnd; iTrack <= l; ++iTrack)
		{
			var track = tracks.ItemByPlayOrder(iTrack);

			var item = new MenuItem();
			item.title = this.getTrackInfo(track);
			if (iTrack == currentPlayOrderIndex)
				item.enabled = false;

			with({track:track})
			{
				item.onSelect = function(){
					if (!track)
					{
						logger.warn('The selected track has gone');
						return;
					}
					logger.debug(track.Name);
					track.Play();
					GetItunes().updatePlayerStatus();
					GetItunes().updateTrackInfoView();
				};
			}

			items.push(item);
		}
		// logger.debug(items.length);
		popupMenu(items, system.event.hOffset, system.event.vOffset);
	}

	// windows only
	// check and alert.  very slow!! (should use runCommandInBg)
	//
	// uses netstat
	// use handle.exe and nkf to see track titles (very slow)
	// http://www.microsoft.com/technet/sysinternals/ProcessesAndThreads/Handle.mspx
	// http://www.vector.co.jp/soft/win95/util/se295331.html
	,checkSomeoneListening: function()
	{
		var hosts = GetConnectedHosts();
		if (!hosts.length)
		{
			ShareTunes.msgbox(widget.getLocalizedString('No one is connecting to you.'));
			return;
		}

		var s = widget.getLocalizedString('Connected from ${0} hosts.').replace(/\$\{0\}/g, hosts.length) + "\n";
		s += hosts.join("\n") + "\n\n";

		if (!filesystem.itemExists(GetItunes().handle) || !filesystem.itemExists(GetItunes().nkf)) {
			s += (widget.getLocalizedString('You need ${handle} and ${nkf} to get track information.')
					.replace(/\${handle}/g, convertPathToPlatform(GetItunes().handle))
					.replace(/\${nkf}/, convertPathToPlatform(GetItunes().nkf))
				+ '\n'
				+ 'http://www.microsoft.com/technet/sysinternals/ProcessesAndThreads/Handle.mspx\n'
				+ 'http://www.vector.co.jp/soft/win95/util/se295331.html'
				);
		}
		else
		{
			var tracksInfo = GetListenedTracksInfo();
			s += tracksInfo.length > 0 
				? widget.getLocalizedString('Someone is listening to') + '\n' + tracksInfo.join('\n')
				: widget.getLocalizedString('No one is listening to tracks.');
		}
		ShareTunes.msgbox(s);

		return;

		// needs netstat
		function GetConnectedHosts()
		{
			var hosts = []; // hash to look up, array to get length and values
			var a = runCommand('netstat').split('\n');
			for (var i = 0, l = a.length; i < l; ++i)
			{
				//  TCP    192.168.11.2:3689      192.168.11.21:1564     ESTABLISHED
				// do not check ESTABLISHED for now
				var line = a[i];
				var matched = line.match(/^\s+TCP\s+[^:]+:3689\s+([^:]+):\d+\s+/);
				if (!matched) continue;
				var host = matched[1];

				if (!hosts[host])
				{
					hosts.push(host);
					hosts[host] = 1;
				}
				else
					hosts[host] += 1;
			}
			return hosts;
		} // GetConnectedHosts

		// handle.exe and nkf.exe
		function GetListenedTracksInfo()
		{
			var libraryXml = filesystem.readFile(GetItunes().getCom().LibraryXMLPath);

			// <key>Music Folder</key>
			// <string>file://localhost/D:/Documents%20and%20Settings/twk/My%20Documents/My%20Music/iTunes/iTunes%20Music/</string>
			// better than system.userMusicFolder;
			// need to remove the last '/'
			var musicFolder = convertPathFromUri(libraryXml.match(new RegExp('<key>Music Folder</key><string>([^<]*)</string>'))[1]);
			if (musicFolder[musicFolder.length - 1] == '/') musicFolder = musicFolder.substr(0, musicFolder.length - 1);
			// logger.debug(musicFolder);

			var doc = XMLDOM.parse(libraryXml);
			var tracks = doc.evaluate('/plist/dict/key[text()="Tracks"]').item(0).nextSibling;
			// widget.dtest = tracks;

			// handle outputs shift-jis strings but the script only reads utf8
			var cmdLine = GetItunes().handle + ' -p iTunes.exe \'' + convertPathToPlatform(musicFolder) + '\''
					+ ' | ' + GetItunes().nkf + ' --utf8';
			// logger.debug(cmdLine);
			var a = runCommand(cmdLine).split('\n');
			// logger.debug(a.length + ' lines returned');
			var tracksInfo = [];
			for (var i = 0, l = a.length; i < l; ++i)
			{
				// logger.debug(a[i]);
				if (!a[i]) continue;
				var matched = a[i].match(/^iTunes.exe[^:]*:[^:]*:\s*(.*)$/);
				if (!matched) continue;

				var localLocation = convertPathToItunesTrackLocation(matched[1]);
				var uriLocation = convertPathToUri(matched[1]).replace(/%E3%80%9C/g, '%EF%BD%9E');;
				// logger.debug("Someone is listening to " + trackLocation);

				try
				{
					// very slow
					var expr = 'dict/string[text()="' + uriLocation + '" or text()="' + localLocation + '"]/..';
					// logger.debug(expr);
					var track = tracks.evaluate(expr).item(0);

					// next-sibling / preceding-sibling does not work correctly (opposite direction?)
					var trackTitle = track.evaluate('key[text()="Name"]').item(0).nextSibling.firstChild.data;
					var trackArtist = track.evaluate('key[text()="Artist"]').item(0).nextSibling.firstChild.data;

					if (trackTitle == GetItunes().getYahoo().trackTitle 
						&& trackArtist == GetItunes().getYahoo().trackArtist)
					{
						continue;
					}

					tracksInfo.push(trackTitle + ' / ' + trackArtist);
				}catch(e){
					logger.warn('cannot find the track.  matched=' + matched);
					continue;
				}
			}
			return tracksInfo;
		} // GetListnedTracksInfo

		// I do not know why such a format...
		// the last replace is for wave marks
		function convertPathToItunesTrackLocation(myPath)
		{
			var trackLocation = encodeURIComponent(myPath.replace(/\\/g, '/')).replace(/%3A/, ':')
				.replace(/%E3%80%9C/g, '%EF%BD%9E');
			var pos = trackLocation.lastIndexOf('%2F');
			return trackLocation.substr(0, pos) + '/' + trackLocation.substr(pos + 3);
		}
		function convertPathForUnixCommand(myPath)
		{
			return myPath.replace(/\\/g, '/');
		}
		function convertPathFromUri(uri)
		{
			return decodeURI(uri).replace(/^file:\/\/localhost\//, '');
		}
		function convertPathToUri(myPath)
		{
			return encodeURI('file://localhost/' + convertPathForUnixCommand(myPath));
		}
	} // checkSomeoneListening

	,searchLyrics: function()
	{
		// Non-Japanese artists are mostly registered as in Katakana name
		// so we search by the track title only
		// var asciiArtist if (this.getYahoo().trackArtist.match(/^[\x00-\x7f]*$/));
		var searchTarget = {SONG:'ss', FULL:'fs'}.SONG;
		var searchPhrase = this.getYahoo().trackTitle;
		var url = 'http://m.search.music.yahoo.co.jp/bin/lyricssearch?cp=${cp}&cc=${cc}&submit2.x=0&submit2.y=0'
			.replace('${cc}', searchTarget)
			.replace('${cp}', escape(searchPhrase));

		this.openUrl(url);
	}

	,searchAmazon: function()
	{
		var url = this.amazon.getOpenUrl(this.getYahoo().trackArtist, this.getYahoo().trackAlbum);
		this.openUrl(url);
	}

	,openWikipedia: function()
	{
		var searchPhrase = this.getYahoo().trackArtist;
		var url = widget.locale == 'ja'
			? 'http://ja.wikipedia.org/wiki/'
			: 'http://en.wikipedia.org/wiki/';
		url += escape(searchPhrase);
		this.openUrl(url);
	}

	,openTwitter : function(){
		if (!this.twitter)
		{
			this.openUrl('http://twitter.com/home');
			return;
		}

		var message = prompt(
				widget.getLocalizedString('What are you doing?'),
				this.twitterAutoUpdate ? '' : this.getTwitterUpdateMessage(),
				'Twitter', 
				widget.getLocalizedString('Go Twitter'), 
				widget.getLocalizedString('Cancel')
			);
		if (message == undefined || message === null)
			return;
		else if (message.length == 0)
		{
			// ok with empty message
//			if (system.platform == SYSTEM_PLATFORM.WINDOWS)
//			{
				if (!widget.twitterWindow)
					this.createTwitterWindowAndFrame();
				else
					animator.start(new FadeAnimation(widget.twitterWindow, 255, 300, animator.kEaseNone));
//			}
//			else
//				this.openUrl('http://twitter.com/home');
		}
		else
			this.twitter.update(message);
	} // openTwitter

	// @return frame
	,createTwitterWindowAndFrame : function ()
	{
		if (widget.twitterWindow)
		{
			logger.info('widget window already exists');
			animator.start(new FadeAnimation(widget.twitterWindow, 255, 300, animator.kEaseNone));
			return;
		}

		var statusesFrame = null;

		var windowWidth = 200;
		var statusesHeight = 200;

		// Create TwitterWindow
		widget.twitterWindow = new Window();
		widget.twitterWindow.title = mainWindow.title; // used as the context menu title for the about box
		widget.twitterWindow.width = windowWidth;
		widget.twitterWindow.hOffset = mainWindow.hOffset + trackInfoFrame.hOffset;
		widget.twitterWindow.vOffset = mainWindow.vOffset + mainWindow.height;
		
		var windowFrame = new Frame();
		windowFrame.window = widget.twitterWindow;
		windowFrame.width = widget.twitterWindow.width;
		
		statusesFrame = new Frame();
		statusesFrame.window = widget.twitterWindow;
		statusesFrame.width = windowWidth;
		statusesFrame.height = statusesHeight;
		statusesFrame.vOffset = 20;
		windowFrame.addSubview(statusesFrame);

		var scrollBar = new ScrollBar();
		scrollBar.window = widget.twitterWindow;
		scrollBar.height = statusesHeight;
		scrollBar.hOffset = windowWidth - scrollBar.width;
		scrollBar.vOffset = statusesFrame.vOffset;
		// same settings as volumeControl
		scrollBar.opacity = 230;
		scrollBar.thumbColor = GetItunes().getColor();
		scrollBar.zOrder = 10000;
		statusesFrame.vScrollBar = scrollBar;
		windowFrame.addSubview(scrollBar); // another frame than statusesFrame

		var statusInput = new TextArea();
		statusInput.lines = 1;
		statusInput.width = widget.twitterWindow.width - 20;
		statusInput.height = 20;
		statusInput.size = 12;
		statusInput.window = widget.twitterWindow;
		statusInput.editable = true;
		statusInput.bgColor = '#ffffff';
		statusInput.bgOpacity = 180;
		statusInput.onKeyPress = function(){
			if (system.event.keyString != 'Return')
				return;
			var message = this.data;
			this.data = '';
			this.rejectKeyPress();

			// sync update - very slow!
			GetItunes().twitter.update(message);
			GetItunes().updateTwitterView();
		};
		windowFrame.addSubview(statusInput);

		var closeButton = new Image();
		closeButton.src = ShareTunes.getImageSrc('closeWindow.png');
		closeButton.onMouseUp = function(){
			animator.start(new FadeAnimation(widget.twitterWindow, 0, 500, animator.kEaseNone));
		};
		closeButton.hOffset = windowFrame.width - 20;
		closeButton.colorize = GetItunes().getColorize();
		windowFrame.addSubview(closeButton);


		var twitterWindowUpdateIntervalMin = 3;

		this.updateTwitterView();
		animator.start(new FadeAnimation(widget.twitterWindow, 255, 300, animator.kEaseNone));

		setInterval(function(){
				// logger.debug('twitter SetInterval fired');
				GetItunes().updateTwitterView();
			}, twitterWindowUpdateIntervalMin * 60 * 1000);

		return statusesFrame;
	} // createTwitterWindowAndFrame

	,removeDeadTracks : function()
	{
		if (!this.getCom()) return;

		var	libraryTracks = this.getCom().LibraryPlaylist.Tracks;
		var libraryTracksCount = libraryTracks.Count

		var msgConfirmStart
			= widget.getLocalizedString("Look for dead tracks from ${0} tracks? (It may take a few minutes)")
				.replace('${0}', libraryTracksCount);
		if (!confirm(msgConfirmStart))
			return;

		var deletedTracksCount = 0;
		for (var i = libraryTracksCount; i > 0; --i)
		{
			var track = libraryTracks.Item(i);
			if (!track)
			{
				logger.debug("track:" + track);
				continue;
			}
			if (track.Kind == ITTrackKind.File && track.Location == "")
			{
				logger.info(track.Name + ' is going to be deleted');
				track.Delete();
				++deletedTracksCount;
			}
		}
		
		ShareTunes.msgbox(deletedTracksCount == 0
			? widget.getLocalizedString('No dead tracks are found.')
			: widget.getLocalizedString('${0} track(s) has been removed from the play list.')
				.replace('${0}', deletedTracksCount)
		);
	}

	,updateViews : function() {
		// logger.debug('updateViews');
		if (system.cpu.idle < 10) return;

		suppressUpdates();
		this.updateVisibilities();
		this.updateTrackInfoView();
		this.updatePlayerStatus();
		this.updateRepeatView();
		this.updateMuteView();
		this.updateVolumeView();
		this.updateShuffleView();
		this.updateRatingView();

		resumeUpdates();
	}
	,updatePlayerStatus : function() 
	{
		// one of stopped, paused, playing, fast forwarding, rewinding and unknown
		var playerStatus = this.getYahoo().playerStatus;
		playPause.src = ShareTunes.getImageSrc(playerStatus == 'playing' ? 'pause.png' : 'play.png');
		playPause.tooltip = playerStatus == 'playing' ? '||' : '>';
	} // updatePlayerStatus
	,updateRepeatView : function() {
		var imgSrcMap = {
			off: ShareTunes.getImageSrc('repeatOff.png')
			,all: ShareTunes.getImageSrc('repeatAll.png')
			,one: ShareTunes.getImageSrc('repeatOne.png')
			};
		repeat.src = imgSrcMap[this.getRepeatMode()] || imgSrcMap['off'];
	}
	,updateVolumeView : function(){
		if (volumeControl.value != 100 - this.getYahoo().volume);
			volumeControl.value = 100 - this.getYahoo().volume;
		systemVolumeControl.value = 16 - system.volume;
	}
	,updateMuteView : function() {
		var nextOpacity = system.mute ? 100 : 255;
		if (mute.opacity != nextOpacity)
			animator.start(new FadeAnimation(mute, nextOpacity, 1000, animator.kEaseOut));

		mute.src = ShareTunes.getImageSrc(this.isMute() ? 'mute.png' : 'noMute.png');
	}
	,updateShuffleView : function() {
		shuffle.src = ShareTunes.getImageSrc(this.isShuffle() ? 'shuffle.png' : 'noShuffle.png');
	}
	,updateRatingView : function() {
		var rating = this.getCurrentRating();
		rate1.src = ShareTunes.getImageSrc(rating >= 1 * 20 ? 'rate.png' : 'noRate.png');
		rate2.src = ShareTunes.getImageSrc(rating >= 2 * 20 ? 'rate.png' : 'noRate.png');
		rate3.src = ShareTunes.getImageSrc(rating >= 3 * 20 ? 'rate.png' : 'noRate.png');
		rate4.src = ShareTunes.getImageSrc(rating >= 4 * 20 ? 'rate.png' : 'noRate.png');
		rate5.src = ShareTunes.getImageSrc(rating >= 5 * 20 ? 'rate.png' : 'noRate.png');
	}
	,updateTrackInfoView : function () {
		var trackInfoData = GetItunes().getCurrentTrackInfo() || mainWindow.title;
		trackInfo.data = trackInfoData;
		trackInfo.tooltip = trackInfoData;
	}
	,updateVisibilities : function() {
		playList.visible = this.getCom() ? true : false;
		removeDead.visible = this.getCom() ? true : false;
		twitter.visible = this.twitter ? true : false;
	}

	,updateInfoFromRemote : function (){
		SetOneTimeTimer(function(){GetItunes().amazon.requestInfoAsync();}, 0.8); // bindAsEventListner
		WikipediaCallBack();
		return;

		function WikipediaCallBack()
		{
			// not supported
			if (widget.locale != 'ja')
				return;

			var searchPhrase = GetItunes().getYahoo().trackArtist;
			if (!searchPhrase)
				return;

			logger.info('Searches Wikipedia by ' + searchPhrase);
			var url = new URL();
			url.location = 'http://wikipedia.simpleapi.net/api?output=json&keyword=' + escape(searchPhrase);
			GetItunes().wikipediaCallBack = function(url){
				log('fetch to ' + url.location + ' is done by response ' + url.response); // + ' ' + url.result);
				var result = url.result;
				var artistInfo = {};
				try {
					if (result.substr(0, 3) == '[{"')
					{
						var json = eval(result);
						if (json.length > 0) artistInfo = json[0];
						else logger.info('not found in wikipedia :' + searchPhrase);
					}
				} catch(e) {
					logger.warn(e);
				};

				GetItunes().wikipediaInfo = artistInfo.strict
					? trim(artistInfo.body.replace(/<br\/>/g, '')) : '';
				GetItunes().updateWikipediaView();
				delete GetItunes().wikipediaCallBack;
			};
			// 3.1.6 engine needs a global reference. 4.0 doesn't.
			url.fetchAsync(GetItunes().wikipediaCallBack);
		} // WikipediaCallBack

		function SetOneTimeTimer(fn, intervalSec, minCpuIdle)
		{
			if (intervalSec == undefined) intervalSec = 1.0;
			if (minCpuIdle == undefined) minCpuIdle = 30;

			var timer = new Timer();
			timer.interval = intervalSec;
			timer.onTimerFired = function(){
				if (system.cpu.idle < minCpuIdle) return;

				timer.ticking = false;
				delete widget[timer.name];
				fn();
			}; // onTimerFired
			timer.ticking = true;
			widget[timer.name] = timer; // this starts ticking
		} // SetWikipediaTimer
	} // updateInfoFromRemote

	,updateAmazonView : function(){
		this._updateButtonView(amazon, GetItunes().amazonInfo);
	}
	,updateWikipediaView : function(){
		this._updateButtonView(wikipedia, GetItunes().wikipediaInfo);
	}

	// @access private
	,_updateButtonView : function(button, info){
		var nextOpacity = null;
		if (info)
		{
			nextOpacity = 255;
			button.onMouseEnter = function(){
				trackInfo.data = info;
			}
			button.onMouseExit = GetItunes().updateTrackInfoView;
		}
		else
		{
			nextOpacity = 80;
			button.onMouseEnter = null;
			button.onMouseExit = null;
		}
		if (button.opacity != nextOpacity)
			animator.start(new FadeAnimation(button, nextOpacity, 1000, animator.kEaseOut));
	}

	// fix duration and delete and addition.  
	// not effective but easy for quick implementation
	,updateTwitterView : function ()
	{
		if (!this.twitter)
		{
			if (widget.twitterWindow)
				animator.start(new FadeAnimation(widget.twitterWindow, 0, 500, animator.kEaseNone));
			return;
		}

		if (!widget.twitterWindow)
			return;

		var statusesFrame = widget.twitterWindow.root.subviews[0].subviews[0];

		if (!statusesFrame.window.visible || statusesFrame.window.opacity == 0)
			return;

		// closeButton and scrollBar
		widget.twitterWindow.root.subviews[0].subviews[2].colorize = GetItunes().getColorize();
		widget.twitterWindow.root.subviews[0].subviews[3].thumbColor = GetItunes().getColor();

		var subviews = statusesFrame.subviews;
		if (subviews)
		{
			// only remove Frame objects
			for (var i = subviews.length - 1; i >= 0; --i)
			{
				if (subviews[i].constructor != statusesFrame.constructor) continue;
				subviews[i].removeFromSuperview();
			}
		}

		// fixed duration...
		var since = new Date(new Date().getTime() - 1000 * 3600 * 24 * 15);
		var MAX_STATUS_COUNT = 20;

		var statuses = GetItunes().twitter.friends_timeline(since);
		for (var i = 0, l = Math.min(statuses.length, MAX_STATUS_COUNT); i < l; ++i)
		{
			var status = statuses[i];
			// logger.debug(status.text + ' by ' + status.user.screen_name);

			var statusFrame = CreateStatusFrame(status);
			statusFrame.vOffset = i * statusFrame.height;
			statusesFrame.addSubview(statusFrame);
		}
		
		return;

		function CreateStatusFrame(status)
		{
			var hMargin = 3;

			var statusFrame = new Frame();
			statusFrame.width = widget.twitterWindow.width;
			statusFrame.height = 30;

			var image = new Image();
			image.width = statusFrame.width;
			image.height = statusFrame.height;
			image.src = ShareTunes.getImageSrc('1.png');
			image.colorize = GetItunes().getColor(); // since 1.png is for colorize
			image.opacity = 240;
			statusFrame.addSubview(image);

			var userProfileImage = new Image();
			userProfileImage.src = status.user.profile_image_url;
			userProfileImage.width = 24;
			userProfileImage.height = 24;
			statusFrame.addSubview(userProfileImage);

			var userName = new Text();
			userName.data = status.user.screen_name;
			userName.size = 9;
			userName.width = 30;
			userName.hOffset = userProfileImage.hOffset + userProfileImage.width + hMargin;
			userName.vOffset = userName.size;
			userName.color = '#0000ff';
			userName.onMouseUp = function(){GetItunes().openUrl('http://twitter.com/' + status.user.screen_name);};
			statusFrame.addSubview(userName);

			var createdAtDate = new Date(status.created_at);
			var createdAt = new Text();
			createdAt.data = 'M/d h:mm:ss'
				.replace('M', createdAtDate.getMonth() + 1)
				.replace('d', createdAtDate.getDate())
				.replace('h', createdAtDate.getHours())
				.replace('mm', (createdAtDate.getMinutes() < 9 ? '0' : '') + createdAtDate.getMinutes())
				.replace('ss', (createdAtDate.getSeconds() < 9 ? '0' : '') + createdAtDate.getSeconds());
			createdAt.size = 9;
			createdAt.hOffset = userName.hOffset + userName.width + hMargin;
			createdAt.vOffset = createdAt.size;
			createdAt.onMouseUp = function(){
				GetItunes().openUrl('http://twitter.com/' + status.user.screen_name + '/statuses/' + status.id);
			};
			statusFrame.addSubview(createdAt);

			var text = new Text();
			text.data = status.text;
			text.tooltip = status.text;
			text.size = 10;
			text.hOffset = userProfileImage.hOffset + userProfileImage.width + hMargin;
			text.vOffset = userName.vOffset + text.size;
			text.width = image.width - userProfileImage.width;
			text.height = userProfileImage.height;
			text.scrolling = 'off';
			text.onMouseEnter = function(){
				this.truncation = 'none';
				this.scrolling = 'autoLeft';
			};
			text.onMouseExit = function(){
				this.truncation = 'end';
				this.scrolling = 'off';
			};
			var urlInText = status.text.match(/(?:ftp|https?):\/\/[^\s]+/);
			if (urlInText)
			{
				text.tooltip = urlInText;
				text.color = '#0000ff';
				text.onMouseUp = function(){GetItunes().openUrl(urlInText);};
			}
			statusFrame.addSubview(text);

			return statusFrame;
		} // CreateStatusFrame
	} // updateTwitterView


	// event Handlers
	,handleOnPlayerPlayEvent: function(track){
		logger.info( "Started to play " + track.Name );

		if (this.twitterAutoUpdate && track != this.lastTrack)
		{
			this.twitter.update(this.getTwitterUpdateMessage(track));
			this.updateTwitterView();
		}

		if (this.lastTrack != track)
		{
			this.lastTrack = track;
			// let duplicate the same track in the history
			this.playHistory[this.playHistoryPointer++] = track;
			if (this.playHistoryPointer >= this.maxTracksInfoCount)
				this.playHistoryPointer = 0;
			// logger.debug(this.playHistoryPointer);
			// for(var i =0;i<this.playHistory.length;++i) logger.debug(this.playHistory[i].Name);
		}

		this.updatePlayerStatus();
		this.updateTrackInfoView();
		this.updateShuffleView();
		this.updateRepeatView();
		this.updateRatingView();
		this.updateInfoFromRemote();
	}
	,handleOnPlayerStopEvent: function(track){
		logger.info( "Stopped to play " + track.Name );
	}
	/** volume change, mute change and start to play */
	,handleOnSoundVolumeChangedEvent : function(newVolume){
		if (GetItunes().isMute())
			logger.info( "Volume changed to mute from " + newVolume + ',' + volumeControl.value);
		else
			logger.info( "Volume changed to " + newVolume + ":" + volumeControl.value);

		this.updateVolumeView();
		this.updateMuteView();
	}
	,getColorize : function() {
		try
		{
			return preferences.useCustomColor.value == '1' ? preferences.color.value : '';
		}
		catch(e)
		{
			log(Object.toJSON(e));
			return '';
		}
	}
	,getColor : function() {
		return this.getColorize() || '#6fa26f';
	}
	,handleOnPreferenceChanged : function(){
		logger.info('handleOnPreferenceChanged');

		// first time load
		if (!preferences.lastPreferenceSaved.value)
		{
			//
		}

		(function NormalizedPreferences()
		{
			var value = preferences.webBrowser.value;
			preferences.webBrowser.value = filesystem.itemExists(value) && !filesystem.isDirectory(preferences.webBrowser.value)
				? value : '';

//			value = preferences.handle.value;
//			preferences.handle.value = filesystem.itemExists(value) && !filesystem.isDirectory(preferences.handle.value)
//				? value : '';

			value = parseInt(preferences.maxTracksInfoCount.value);
			preferences.maxTracksInfoCount.value = !isNaN(value) ? value : GetItunes().maxTracksInfoCount;

			value = parseInt(preferences.updateViewsFrequenceSec.value);
			preferences.updateViewsFrequenceSec.value = !isNaN(value) ? value : GetItunes().updateViewsFrequenceSec;
		})();

		['mainImage', 'backTrack', 'playPause', 'nextTrack', 
			'playList', 'showItunes', 'twitter',
			'rate1', 'rate2', 'rate3', 'rate4', 'rate5', 
			'shuffle', 'repeat', 'settings', 'removeDead',
			'lyrics', 'amazon', 'wikipedia', 'mute'].each(function(id){
			$(id).colorize = GetItunes().getColorize();
		});
		['volumeControl', 'systemVolumeControl'].each(function(id){
			$(id).thumbColor = GetItunes().getColor();
		});
		try
		{
			ShareTunes.msgbox.msg.color = 
			ShareTunes.msgbox.msg2.data = "\n" + msg + "\n"; // add paddings
		} catch (e) {
			// maybe not initialized yet
		}

		this.webBrowser = preferences.webBrowser.value;
//		this.handle = preferences.handle.value;
		this.hideItunesOnGainFocus = preferences.hideItunesOnGainFocus.value != '0';
		// logger.debug(preferences.hideItunesOnGainFocus.value + ':' + this.hideItunesOnGainFocus);
		this.maxTracksInfoCount = parseInt(preferences.maxTracksInfoCount.value);

		if (this.maxTracksInfoCount > this.playHistory.length)
		{
			// resize the play history accodring to the new max count
			this.playHistory = this.playHistory.slice(this.playHistoryPointer)
				.concat(this.playHistory.slice(0, this.playHistoryPointer))
				.slice(0, this.maxTracksInfoCount);
			this.playHistoryPointer = 0;
		}

		this.updateViewsFrequenceSec = parseInt(preferences.updateViewsFrequenceSec.value);
		updateViewsTimer.interval = this.updateViewsFrequenceSec;
		updateViewsTimer.ticking = this.updateViewsFrequenceSec > 0;

		if ('0' == preferences.twitterUsesTwitter.value)
		{
			logger.info('Unbinding twitter');
			preferences.twitterAutoUpdate.value = '0';
			this.twitter = null;
			this.twitterAutoUpdate = false;
		}
		else
		{
			// TODO should check the current user instead of always assignment
			this.twitter = new Twitter(
				preferences.twitterUserName.value,
				preferences.twitterPassword.value);
			this.twitterAutoUpdate = preferences.twitterAutoUpdate.value == '1';
		}

		preferences.lastPreferenceSaved.value = new Date().getTime();

		this.updateTwitterView();
		this.updateVisibilities();
	}
	,handleOnUnLoad : function(){
		// if (this.getCom()) this.getCom().Quit(); // not stable
		this.releaseCom();
	}
	,handleOnGainFocus : function(){
		// logger.debug('ongain');
		if (this.hideItunesOnGainFocus)
			this.hideItunes();
	}
	,handleOnDragDrop : function(){
		if (!system.event.data) return;
		if (system.event.data[0] != 'filenames') {
			logger.debug(system.event.data);
			return;
		}

		var filename = system.event.data[1];
		// var filename = convertPathToPlatform(system.event.data[1]);
		logger.debug(filename);

		if (!filesystem.itemExists(filename)) return;

		// Just pass the path to iTunes.  Even a directory is droppable
		if (GetItunes().getCom())
			GetItunes().getCom().PlayFile(filename);
		else
		{
			var pos = filename.lastIndexOf('.');
			if (!pos) return;
			var ext = filename.substr(pos + 1).toLowerCase();
			if ({'mp3':1, 'aiff':1, 'au':1, 'wav':1, 'snd':1}[ext])
				play(filename, true);
		}
	}

	,handleTrackInfoOnMultiClick : function(){
		try
		{
			var info = this.getCurrentTrackDetailInfo();
			if (!info) return;

			ShareTunes.msgbox(info, function(){
				if (GetItunes().getCom())
					ShowArtwork(GetItunes().getCom().CurrentTrack);
			});
		}
		catch (e)
		{
			// may thrown TypeError if currentTrack is changed at the moment
			logger.warn(e);
		}
		return;

		// show the first artwork
		// @throw TypeError if currentTrack is changed at the moment
		function ShowArtwork(currentTrack)
		{
			var artWorksFolder = system.widgetDataFolder + '/ArtWorks';
			if (!filesystem.isDirectory(artWorksFolder))
				mkdir(artWorksFolder);

			var artWorkFileName = artWorksFolder + '/' + currentTrack.trackID ;

			var savedArtWork = false;
			if (filesystem.itemExists(artWorkFileName))
				savedArtWork = true;
			else
			{
				// only the first artWork
				var artWorks = currentTrack.Artwork
				if (artWorks.Count >= 1)
				{
					try {
						artWorks.Item(1).SaveArtworkToFile(convertPathToPlatform(artWorkFileName));
						savedArtWork = true;
					} catch(e) {
						logger.warn(e);
					}
				}
			}

			if (savedArtWork)
			{
				logger.debug(artWorkFileName);
				if (!widget.artWork)
				{
					var win = new Window();
					win.level = 'topmost';
					widget.artWork = new Image();
					widget.artWork.window = win;
					widget.artWork.onMouseDown = function(){
						animator.start(new FadeAnimation(widget.artWork.window, 0, 500, animator.kEaseNone));
					};
				}
				widget.artWork.src = artWorkFileName;
				animator.start(new FadeAnimation(widget.artWork.window, 255, 500, animator.kEaseNone));
			}
		} // ShowArtwork
	}
	,handleBackTrackOnMouseEnter : function(){
		var trackInfoData = this.getBackTrackInfo();
		if (trackInfoData) trackInfo.data = trackInfoData;
	}
	,handleBackTrackOnMouseExit : function(){
		this.updateTrackInfoView();
	}
	,handleBackTrackOnMouseDown : function(){
		mouseEventTimer.onTimerFired = function(){
			mouseEventTimer.ticking = false;
			if (new Date().getTime() - system.event.timestamp.getTime() > GetItunes().popupMouseDownMsec)
			{
				GetItunes().popupPlayHistoryInfo() || GetItunes().handleBackTrackOnMouseUp();
			}
		}; // onTimerFired
		mouseEventTimer.ticking = true;
	}
	,handleBackTrackOnMouseUp : function(){
		mouseEventTimer.ticking = false;
		GetItunes().backTrack();
		GetItunes().updatePlayerStatus();
		GetItunes().updateTrackInfoView();
	}

	,handleNextTrackOnMouseEnter : function(){
		var trackInfoData = this.getNextTrackInfo();
		if (trackInfoData) trackInfo.data = trackInfoData;
	}
	,handleNextTrackOnMouseExit : function(){
		this.updateTrackInfoView();
	}
	,handleNextTrackOnMouseDown : function(){
		mouseEventTimer.onTimerFired = function(){
			mouseEventTimer.ticking = false;
			if (new Date().getTime() - system.event.timestamp.getTime() > GetItunes().popupMouseDownMsec)
			{
				GetItunes().popupNextTracksInfo() || GetItunes().handleBackTrackOnMouseUp();
			}
		}; // onTimerFired
		mouseEventTimer.ticking = true;
	}
	,handleNextTrackOnMouseUp : function(){
		mouseEventTimer.ticking = false;
		GetItunes().nextTrack();
		GetItunes().updatePlayerStatus();
		GetItunes().updateTrackInfoView();
	}

	,handleSettingsOnMouseUp : function(){
		if (system.event.modifiers == 'option')
			this.handleSettingsOnMultiClick();
		else
			showWidgetPreferences();
	}
	,handleSettingsOnMultiClick : function(){
		if (!confirm(widget.getLocalizedString('Clear all settings?')))
			return;
		
		for (var name in preferences) preferences[name].value = '';
		savePreferences();
		reloadWidget();
	}
	,handleOnRunCommandInBgComplete : function(){
		logger.info("onRunCommandInBgComplete for tag: " + system.event.data);
	}
	,handleEvent : function(triggerName){
		// logger.debug(triggerName);
	}
	,handleOnKonsposeActivated : function(){
		// sometimes the widget engine does not set focus so we do by myself
		mainWindow.focus();
	}
	,handleOnWakeFromSleep : function(){
		// one reason to reload is to update the program
		// another reason is perhaps COM or some stability
		// we lose play history
		// reloadWidget();
	}
	,handleOnTellWidget : function(){
		logger.debug(system.event.data);
	}
} // ShareTunes.prototype
// @param relativePath filename.png
// @return CONTENTS_FOLDER/Images/filename.png
ShareTunes.getImageSrc = function(relativePath){
	return CONTENTS_FOLDER + '/Images/' + relativePath
};
// show an alert message dialog
// @param msg string
// @param callback optional function
ShareTunes.msgbox = function(msg, callback)
{
	if (system.platform == SYSTEM_PLATFORM.MACINTOSH)
	{
		alert(msg);
		callback();
		return;
	}

	if (!callback) callback = function(){};

	var dialog = ShareTunes.msgbox.window;
	if (!ShareTunes.msgbox.window)
	{
		// fixed settings

		dialog = ShareTunes.msgbox.window = new Window();
		dialog.title = mainWindow.title;
		dialog.level = 'topmost';
		// dialog.onLoseFocus =
		// dialog.onMultiClick = 
		dialog.onContextMenu 
			= function(){dialog.visible = false; callback();};

		var frame = new Frame();

		var titleBar = new Text();
		titleBar.vOffset = 12;
		titleBar.height = 16;
		titleBar.size = 12;
		titleBar.bgOpacity = 170;
		titleBar.data = ' ' + dialog.title + ' '; // add paddings
		frame.addSubview(titleBar);
		ShareTunes.msgbox.titleBar = titleBar;

		// double textarea
		var message = new TextArea();
		frame.addSubview(message);
		message.editable = false;
		message.scrollbar = false;
		message.vOffset = titleBar.height;
		message.size = 11;
		message.bgColor = '#ffffff';
		message.bgOpacity = 224;
		message.color = '#ffffff';
		message.onMouseUp = function(){dialog.visible = false; callback();};
		ShareTunes.msgbox.msg = message;

		var message2 = new TextArea();
		frame.addSubview(message2);
		message2.editable = false;
		message2.scrollbar = false;
		message2.vOffset = titleBar.height;
		message2.size = 11;
		message2.bgOpacity = 128;
		message2.onMouseUp = function(){dialog.visible = false; callback();};
		ShareTunes.msgbox.msg2 = message2;

		frame.window = dialog;
	}

	// dynamic settings
	ShareTunes.msgbox.titleBar.bgColor = GetItunes().getColor();
	ShareTunes.msgbox.msg2.bgColor = GetItunes().getColor();

	if (msg[msg.length - 1] != "\n") msg += "\n";
	ShareTunes.msgbox.msg.data = "\n" + msg + "\n"; // add paddings
	ShareTunes.msgbox.msg2.data = "\n" + msg + "\n"; // add paddings

	dialog.visible = true;
	dialog.focus();
	beep();
}; // msgbox
// ShareTunes static functions

// using asamashi
// http://groups.yahoo.co.jp/group/YJDN-search/message/74
// http://affiliate.amazon.co.jp/gp/associates/network/help/t5/ref=amb_link_10130306_2/249-5586910-6897900
function Amazon(tag, accessKey){
	this.tag = tag;
	this.accessKey = accessKey;
} // Amazon
// @return the real album title, not including '(Disc 1)' or something like that
Amazon.getAlbumName = function(trackAlbum)
{
	return trackAlbum.replace(/\s+[(\[]?Disc \d+[)\]]?$/i, '');
};
Amazon.prototype.getOpenUrl = function (trackArtist, trackAlbum)
{
	var url = null;

	if (trackArtist != '')
	{
		// index might be music-jp or blended?
		var searchPhrase = '"' + Amazon.getAlbumName(trackAlbum) + '" "' + trackArtist + '"';
		if (!this.tag) logger.error('unexpected state : Amazon#getOpenUrl');
		url = this.tag
			? ('http://www.amazon.co.jp/gp/search'
				+ '?ie=UTF8&linkCode=ur2&camp=247&creative=1211&tag=' + this.tag
				+ '&keywords=' + escape(searchPhrase) + '&index=music-jp')
			: ('http://www.amazon.co.jp/s/ref=nb_ss_gw/503-2750863-4428735'
				+ '?__mk_ja_JP=%83J%83%5E%83J%83i&url=search-alias%3Daps'
				+ '&field-keywords=${field-keywords}&Go.x=0&Go.y=0&Go=Go'
				).replace('${field-keywords}', EscapeSJIS(searchPhrase));
	}
	else
	{
		// top
		url = widget.locale != 'ja'
			? 'http://www.amazon.com/'
			: this.tag 
			? ('http://www.amazon.co.jp/gp/redirect.html'
				+ '?ie=UTF8&linkCode=ur2&camp=247&creative=1211&tag=' + this.tag
				+ '&location=http%3A%2F%2Fwww.amazon.co.jp%2F')
			: 'http://www.amazon.co.jp/';
	}

	return url;

	// dummy function
	function EscapeSJIS(s){logger.error('unimplemented EscapedSJIS is called'); return escape(s);}
}; // getOpenUrl
Amazon.prototype.requestInfoAsync = function()
{
	// not supported
	if (widget.locale != 'ja')
		return;

	var albumName = Amazon.getAlbumName(GetItunes().getYahoo().trackAlbum);
	if (!albumName)
		return;

	var searchPhrase = '"' + albumName + '" "' + GetItunes().getYahoo().trackArtist + '"';
	logger.info('Searches Amazon by ' + searchPhrase);
	var url = 'http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService';
	url += '&AWSAccessKeyId=' + this.accessKey + '&AssociateTag=' + this.tag;
	url += '&Operation=ItemSearch&SearchIndex=Music&ResponseGroup=Small,OfferSummary&ItemPage=1';
	url += '&Availability=Available&Condition=All&MerchantId=All';
	url += '&Keywords=' + escape(searchPhrase);
	//logger.debug('Searches Amazon with ' + url);

	var xhr = new XMLHttpRequest();
	xhr.onreadystatechange = ItemSearchOnReadyStateChange;
	xhr.open('GET', url, true);
	xhr.send();

	// abort if late to respond
	setTimeout(function(){
		if (xhr.readyState == 4)
			return;

		logger.warn('aborting the request of ' + searchPhrase);
		xhr.abort();
		GetItunes().amazonInfo = '';
		GetItunes().updateAmazonView();
	}, 5000); // setTimeout

	return;

	function ItemSearchOnReadyStateChange()
	{
		if (this.readyState != 4) return; // finished

		// logger.debug(this.responseText);
		GetItunes().amazonInfo = GetInfo(this.responseXML);
		GetItunes().updateAmazonView();
		return;
		
		// @param doc xml
		// @return string
		function GetInfo(doc)
		{
			if (doc.evaluate('count(//CorrectedQuery)') || doc.evaluate('count(//Errors)'))
			{
				logger.info('not found in amazon');
				return '';
			}

			var lowestNewPrice = doc.evaluate('string(//Item[1]/OfferSummary/LowestNewPrice/FormattedPrice)')
				.replace(/\s/g, ''); //.replace(/￥/g, '\\');
			var lowestUsedPrice = doc.evaluate('string(//Item[1]/OfferSummary/LowestUsedPrice/FormattedPrice)')
				.replace(/\s/g, ''); //.replace(/￥/g, '\\');
			if (lowestNewPrice == lowestUsedPrice) lowestUsedPrice = '';

			var title = doc.evaluate('string(//Item[1]/ItemAttributes/Title)');
			var artist = doc.evaluate('string(//Item[1]/ItemAttributes/Artist)');

			var priceInfo = lowestNewPrice;
			if (lowestNewPrice && lowestUsedPrice) priceInfo += '/';
			priceInfo += lowestUsedPrice;
			if (lowestNewPrice || lowestUsedPrice) priceInfo += ' ';
			priceInfo += title + '/' + artist;
			
			return priceInfo;
		} // GetInfo
	} // ItemSearchOnReadyStateChange
}; // requestInfoAsync
// end of Amazon.prototype


/* global functions for COM.connectObject */
function iTunes_OnPlayerPlayEvent( track ){ GetItunes().handleOnPlayerPlayEvent(track); }
function iTunes_OnPlayerStopEvent( track ){ GetItunes().handleOnPlayerStopEvent(track); }
function iTunes_OnSoundVolumeChangedEvent( newVolume ){
	GetItunes().handleOnSoundVolumeChangedEvent(newVolume);
}
function iTunes_OnQuittingEvent(){
	GetItunes().releaseCom();
}
// we need to handle these events not to crash
function iTunes_OnCOMCallsDisabledEvent(reason){
	logger.info('disabledEvent by ' + reason);
	if (!GetItunes().com) return;
	GetItunes().disabledCom = GetItunes().com;
	GetItunes().com = false;
}
function iTunes_OnCOMCallsEnabledEvent(){
	logger.info('enabledEvent');
	if (!GetItunes().disabledCom) return;
	GetItunes().com = GetItunes().disabledCom;
	GetItunes().disabledCom = null;

	GetItunes().updateViews();
}
// [object VBArray] which has no proparties, toArray() not worked
//function iTunes_OnDatabaseChangedEvent(deletedObjectIds, changedObjectIds){
//	logger.debug('deleted=' + deletedObjectIds + ', changed=' + changedObjectIds);
//	var deleteds = new VBArray(deletedObjectIds);
//	var changeds = new VBArray(changedObjectIds);
//	logger.debug(changeds.toArray());
//}
/* end of COM global functions */


// @return ShareTunes main object
function GetItunes()
{
	if (!arguments.callee.shareTunes)
		arguments.callee.shareTunes = new ShareTunes();
	return arguments.callee.shareTunes;
}


function HandleOnLoad(){
	logger.info('handleOnLoad');

	CreatAdditionalKonstruction();
	CreatAdditionalKonstruction = null;

	if (!GetItunes().getYahoo().running)
		GetItunes().runItunes();

	// hide COM depending setting items
	if (!GetItunes().getCom())
	{
		preferences.maxTracksInfoCount.hidden = true;
		preferences.twitterAutoUpdate.hidden = true;
//		preferences.handle.hidden = true;
	}

	// read the preference settings
	GetItunes().handleOnPreferenceChanged();

	GetItunes().updateViews();

	// in non COM environment, there is no way to udpate the initial state
	// so we give up the call to updateInfo
	if (GetItunes().getCom())
		GetItunes().updateInfoFromRemote();

	mainWindow.visible = true;
	animator.start(new FadeAnimation(mainWindow, 255, 500, animator.kEaseNone));
	mainWindow.focus();

	return;

	// things which might be included in .kon
	function CreatAdditionalKonstruction(){
		mainWindow.width = 330;
		mainWindow.height = 80;

		amazon.visible = widget.locale == 'ja';
		lyrics.visible = widget.locale == 'ja';

		// event settings

		mainWindow.onContextMenu = ee(function(){
			var items = [];

			if (widget.updateTimer)
				items.push(CreateUpdateMenuItem());

			// if platform is windows (you can use this in Macintosh with lsof)
			if (GetItunes().handle && GetItunes().nkf && system.platform == SYSTEM_PLATFORM.WINDOWS)
				items.push(CreateSomeoneListeningMenuItem());
			
			items.push(CreateRemoveDeadTracksMenuItem());

			mainWindow.contextMenuItems = items;
			return;

			// @return MenuItem
			function CreateUpdateMenuItem()
			{
				var item = new MenuItem();
				item.title = widget.getLocalizedString('Check For Update');
				item.onSelect = function(){
					if (widget.updateTimer)
						widget.updateTimer.onTimerFired();
					reloadWidget();
				};
				return item;
			} // CreateUpdateMenuItem()
			// show people listening files
			function CreateSomeoneListeningMenuItem()
			{
				var item = new MenuItem();
				item.title = widget.getLocalizedString('Does someone listen to my tracks?');
				item.onSelect = ee(function(){GetItunes().checkSomeoneListening();});
				return item;
			} // CreateSomeoneListeningMenuItem
			function CreateRemoveDeadTracksMenuItem(){
				var item = new MenuItem();
				item.title = widget.getLocalizedString('Remove Dead Tracks...');
				item.onSelect = ee(function(){GetItunes().removeDeadTracks();});
				return item;
			} // CreateRemoveDeadTracksMenuItem
		}); // onContextMenu

		mainFrame.onDragDrop = ee(function(){GetItunes().handleOnDragDrop();});
		mainFrame.onMultiClick = ee(function(){GetItunes().updateViews();});
		trackInfo.onMultiClick = ee(function(){GetItunes().handleTrackInfoOnMultiClick();});


		backTrack.onMouseEnter = ee(function(){GetItunes().handleBackTrackOnMouseEnter();});
		backTrack.onMouseExit = ee(function(){GetItunes().handleBackTrackOnMouseExit();});
		backTrack.onMouseDown = ee(function(){GetItunes().handleBackTrackOnMouseDown();});
		backTrack.onMouseUp = ee(function(){GetItunes().handleBackTrackOnMouseUp();});

		playPause.onMouseUp = ee(function(){
			GetItunes().playPause();
			GetItunes().updatePlayerStatus();
			GetItunes().updateTrackInfoView();
		});

		nextTrack.onMouseEnter = ee(function(){GetItunes().handleNextTrackOnMouseEnter();});
		nextTrack.onMouseExit = ee(function(){GetItunes().handleNextTrackOnMouseExit();});
		nextTrack.onMouseDown = ee(function(){GetItunes().handleNextTrackOnMouseDown();});
		nextTrack.onMouseUp = ee(function(){GetItunes().handleNextTrackOnMouseUp();});

		playList.onMouseDown = ee(function(){GetItunes().selectPlayList();});
		showItunes.onMouseUp = ee(function(){GetItunes().toggleItunes();});

		twitter.onMouseUp = ee(function(){GetItunes().openTwitter();});
		rateFrame.onMouseDown = ee(function(){GetItunes().rate(0);});
		rateFrame.onMultiClick = ee(function(){GetItunes().rate(0);});
		rate1.onMouseDown = ee(function(){GetItunes().rate(20);});
		rate2.onMouseDown = ee(function(){GetItunes().rate(40);});
		rate3.onMouseDown = ee(function(){GetItunes().rate(60);});
		rate4.onMouseDown = ee(function(){GetItunes().rate(80);});
		rate5.onMouseDown = ee(function(){GetItunes().rate(100);});

		shuffle.onMouseDown = ee(function(){
			GetItunes().toggleShuffle('shuffle');
			GetItunes().updateShuffleView();
			GetItunes().updatePlayerStatus();
		});
		repeat.onMouseDown = ee(function(){
			GetItunes().toggleRepeatMode();
			GetItunes().updateRepeatView();
			GetItunes().updatePlayerStatus();
		});

		settings.onMouseUp = ee(function(){GetItunes().handleSettingsOnMouseUp();});
		settings.onMultiClick = ee(function(){GetItunes().handleSettingsOnMultiClick();});

		removeDead.onMouseUp = ee(function(){GetItunes().removeDeadTracks();});

		lyrics.onMouseUp = ee(function(){GetItunes().searchLyrics();});
		amazon.onMouseUp = ee(function(){GetItunes().searchAmazon();});
		wikipedia.onMouseUp = ee(function(){GetItunes().openWikipedia();});

		mute.onMouseUp = ee(function(){GetItunes().toggleMute(); GetItunes().updateMuteView();});

		volumeControl.onValueChanged = ee(function(){GetItunes().setVolume(100 - volumeControl.value);});
		systemVolumeControl.onValueChanged = ee(function(){system.volume = 16 - systemVolumeControl.value;});

		mouseEventTimer.onTimerFired = ee(function(){
			log(system.event.type + ', timestamp=' + system.event.timestamp);
		});
		updateViewsTimer.onTimerFired = ee(function(){GetItunes().updateViews();});

	} // CreatAdditionalKonstruction
} // HandleOnLoad

e(function(){
	HandleOnLoad();
	widget.onLoad = HandleOnLoad = null;
});
