/**
 * Log appenders for Yahoo! Widget
 *
 * YahooWidgetLogAppender logs with log()
 * YahooWidgetPrintAppender logs with print()
 * Builtin JSAlertAppender is also appliable.  ConsoleAppender is not.
 *
 * @author twk
 */

/**
 * Yahoo Widget Log Appender writes the logs with the Yahoo! Widget log() function
 * @constructor
 * @extends Appender  
 * @param logger log4js instance this appender is attached to
 * @author twk
 */
function YahooWidgetLogAppender(logger) {
	// add listener to the logger methods
	logger.onlog.addListener(this.doAppend.bind(this));
	logger.onclear.addListener(this.doClear.bind(this));
	/**
	 * set reference to calling logger
	 * @type Log4js.Logger
	 */
	this.logger = logger;
	this.layout = new SimpleLayout();
}
 
YahooWidgetLogAppender.prototype = {
	/** 
	 * @see Appender#doAppend
	 */
	doAppend: function(loggingEvent) {
		log((this.layout.getHeader() + this.layout.format(loggingEvent) + this.layout.getFooter()
			).replace(/\n$/, ''));
	},
	/** 
	 * @see Appender#doClear
	 */
	doClear: function() {
		return;
	},
	
	/**
	 * @see Appender#setLayout
	 */
	setLayout: function(layout){
		this.layout = layout;
	} 
};


/**
 * Yahoo Widget Print Appender writes the logs with the Yahoo! Widget print() function
 * @constructor
 * @extends Appender  
 * @param logger log4js instance this appender is attached to
 * @author twk
 */
function YahooWidgetPrintAppender(logger) {
	// add listener to the logger methods
	logger.onlog.addListener(this.doAppend.bind(this));
	logger.onclear.addListener(this.doClear.bind(this));
	/**
	 * set reference to calling logger
	 * @type Log4js.Logger
	 */
	this.logger = logger;
	this.layout = new SimpleLayout();
}
 
YahooWidgetPrintAppender.prototype = {
	/** 
	 * @see Appender#doAppend
	 */
	doAppend: function(loggingEvent) {
		print((this.layout.getHeader() + this.layout.format(loggingEvent) + this.layout.getFooter()
			).replace(/\n$/, ''));
	},
	/** 
	 * @see Appender#doClear
	 */
	doClear: function() {
		return;
	},
	
	/**
	 * @see Appender#setLayout
	 */
	setLayout: function(layout){
		this.layout = layout;
	} 
};
